import argparse
import sys
import os

import yaml

from vdg import __version__
from vdg.draftgen import DraftGenerator
from vdg.autopub import BangumiUploader, NyaaUploader
from vdg.utils import read_from_file, write_to_file

def parse_args():
    parser = argparse.ArgumentParser(description="发布稿生成/發佈腳本.")
    parser.add_argument('--version', action='version', version=__version__)
    subparsers = parser.add_subparsers(dest='command', required=True)

    subparsers.add_parser('new', help='在当前目录下生成配置文件 config.yml, 存放截图的文件 url.html 和 url.md, 以及存放 MediaInfo 的 mediainfo.txt.')

    parser_gen = subparsers.add_parser('gen', help='根据在当前目录下的配置文件, 生成发布稿文件.')
    parser_gen.add_argument('--site', type=str, choices=['bangumi', 'nyaa', 'vcb-s', 'all'], help='选择需要生成稿件的站点 (bangumi,nyaa,vcb-s,all).', default='all')

    parser_pub = subparsers.add_parser('publish', help='开启半自动化发布流程.')
    parser_pub.add_argument('-s', '--site', type=str, choices=['bangumi', 'nyaa'], help='选择需要发布的站点 (bangumi or nyaa).', required=True)
    parser_pub.add_argument('-u', '--username', type=str, help='站点用户名.', required=True)
    parser_pub.add_argument('-p', '--password', type=str, help='站点密码.', required=True)

    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    if args.command == 'new':
        DraftGenerator.generate_configs()
    elif args.command == 'gen':
        site = args.site
        release_info = yaml.safe_load(read_from_file("./config.yml"))
        gen = DraftGenerator(release_info)
        sites = ['bangumi', 'nyaa', 'vcb-s'] if site == 'all' else [site]

        for site in sites:
            ext = "md" if site == 'nyaa' else "html"
            filepath = f"./{release_info["filename"]}_{site}.{ext}"
            if os.path.isfile(filepath):
                user_input = input(f"The file '{filepath}' already exists. Do you want to overwrite it? (y/n): ").lower()
                if user_input != 'y':
                    continue

            write_to_file(
                filepath,
                gen.generate(site, "title") + "\n" + gen.generate(site, "draft")
            )

    elif args.command == 'publish':
        site = args.site
        if site == 'bangumi':
            uploader = BangumiUploader()
        elif site == 'nyaa':
            uploader = NyaaUploader()
        release_info = yaml.safe_load(read_from_file("./config.yml"))
        gen = DraftGenerator(release_info)
        uploader.run(
            args.username,
            args.password,
            gen.generate(site, "title"),
            release_info["category"][site],
            gen.generate(site, "draft"),
            release_info["种子文件路径"]
        )



import os
import inspect
from pathlib import Path
import re
import html2text
from jinja2 import Environment, FileSystemLoader
import vdg.filters as filters
from vdg.utils import read_from_file, write_to_file, create_file


class DraftGenerator:

    def __init__(self, release_info):
        self.release_info = release_info

    def __format_text(self, text):
        text = re.sub(r"^\n+", "", text)
        text = re.sub(r"\n{3,}", "\n", text)
        text = re.sub(r"[ \t]+$", "", text, flags=re.MULTILINE)
        text = re.sub(r'\s+$', '', text)
        return text

    def __generate_draft(self, template_name):
        """Generate drafts using the specified Jinja2 template."""
        env = Environment(
            loader=FileSystemLoader(Path(__file__).parent.absolute() / "templates"),
            trim_blocks=True,
            lstrip_blocks=True,
        )

        # Register custom filters
        for name, func in inspect.getmembers(filters, inspect.isfunction):
            env.filters[name] = func

        # Load and render the template
        template = env.get_template(template_name)
        content = self.__format_text(template.render(**self.release_info))
        return content

    def __generate_bangumi_draft(self):
        """Generate the Bangumi draft."""
        return self.__generate_draft("bangumi.html.jinja")

    def __generate_bangumi_title(self):
        """Generate the Bangumi title."""
        return self.__generate_draft("bangumi.title.txt.jinja")

    def __generate_nyaa_draft(self):
        """Generate the Nyaa draft by converting the Bangumi draft to markdown."""
        bangumi_draft = self.__generate_bangumi_draft()
        converter = html2text.HTML2Text(bodywidth=0)
        md_str = converter.handle(bangumi_draft)

        # Append comparison images in markdown format if available
        comparison_md = read_from_file( self.release_info["对比图"]["MD"] )
        if comparison_md:
            md_str = "* * *".join([md_str.rsplit("* * *", 1)[0], "\n" + comparison_md])
        return md_str

    def __generate_vcb_s_draft(self):
        """Generate the VCB-S.com draft."""
        return self.__generate_draft("vcb-s.html.jinja")

    def __generate_vcb_s_title(self):
        """Generate the VCB-S.com title."""
        return self.__generate_draft("vcb-s.title.txt.jinja")

    def generate(self, site, content):
        if site == 'bangumi' and content == "draft":
            return self.__generate_bangumi_draft()
        elif site == 'bangumi' and content == "title":
            return self.__generate_bangumi_title()
        elif site == 'nyaa' and content == "draft":
            return self.__generate_nyaa_draft()
        elif site == 'nyaa' and content == "title":
            return self.__generate_bangumi_title()
        elif site == 'vcb-s' and content == "draft":
            return self.__generate_vcb_s_draft()
        elif site == 'vcb-s' and content == "title":
            return self.__generate_vcb_s_title()
        else:
            raise ValueError("Invalid site or content type.")

    @staticmethod
    def __get_empty_config():
        return read_from_file( Path(__file__).parent.absolute() / "templates/form.yml" )

    @staticmethod
    def generate_configs():
        if os.path.exists("./config.yml"):
            print("WARNING: Config already exists. Override? (y/n) ", end='')
            if input().lower() != "y":
                return
        write_to_file("./config.yml", DraftGenerator.__get_empty_config())
        for filename in ["./url.html", "./url.md", "./mediainfo.txt"]:
            create_file(filename)



def read_from_file(file_path):
    with open(file_path, "r", encoding="utf-8") as f:
        return f.read()

def write_to_file(file_path, content):
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(content)

def create_file(file_path):
    with open(file_path, "w", encoding="utf-8"):
        pass


import re

合作组英文 = {
    "SweetSub": "SweetSub",
    "茉语星梦": "MakariHoshiyume",
    "风之圣殿": "FZSD",
    "千夏字幕组": "Airota",
    "喵萌奶茶屋": "Nekomoe kissaten",
    "诸神字幕组": "Kamigami",
    "天香字幕社": "T.H.X",
    "星空字幕组": "XKsub",
    "白恋字幕组": "Shirokoi",
    "动漫国字幕组": "DMG",
    "悠哈璃羽字幕社": "UHA-WINGS",
}

CJK_PATTERN = r"[\u4E00-\u9FFF\u3400-\u4DBF\u3000-\u303F\u3040-\u309F\u30A0-\u30FF\u31F0-\u31FF\uAC00-\uD7AF]"
ENG_PATTERN = r"[a-zA-Z0-9]"

def enforce_space(text):
    text = re.sub(f"({CJK_PATTERN})({ENG_PATTERN})", r"\1 \2", text)
    text = re.sub(f"({ENG_PATTERN})({CJK_PATTERN})", r"\1 \2", text)
    return text

def add_br(text):
    return text.replace('\n', '<br>\n')


def format_links(text):
    return '\n\n'.join(
        [f'<a href="{link}" rel="noopener" target="_blank">{link}</a>' for link in text.split('\n') if link != ""])


def del_anchors(text):
    return text.replace('<a', '<del><a').replace('</a>', '</a></del>')


def group_name_eng(names):
    return [合作组英文[name] for name in names]


def read_file(file_path):
    with open(file_path, "r", encoding="utf-8") as f:
        return f.read()



import time
import random
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.support.ui import WebDriverWait, Select
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import WebDriverException


class WebAutomationBase:
    def __init__(self):
        self.driver = None

    def random_delay(self, min_delay=0.5, max_delay=1):
        time.sleep(random.uniform(min_delay, max_delay))

    def get_driver(self):
        options = Options()
        options.add_argument("--width=1920")
        options.add_argument("--height=1080")
        profile = webdriver.FirefoxProfile()
        profile.set_preference("general.useragent.override", "Mozilla/5.0 (iPhone14,3; U; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) Version/10.0 Mobile/19A346 Safari/602.1")
        profile.set_preference("dom.webdriver.enabled", False)
        profile.set_preference("useAutomationExtension", False)
        profile.set_preference("privacy.trackingprotection.enabled", True)
        profile.update_preferences()
        options.profile = profile
        self.driver = webdriver.Firefox(options=options)

    def close_driver(self):
        try:
            self.driver.close()
        except WebDriverException:
            pass

    def wait_for_element(self, by, value, timeout=10):
        return WebDriverWait(self.driver, timeout).until(
            EC.presence_of_element_located((by, value))
        )

    def wait_for_clickable(self, by, value, timeout=10):
        return WebDriverWait(self.driver, timeout).until(
            EC.element_to_be_clickable((by, value))
        )

    def wait_for_staleness(self, element, timeout=10):
        WebDriverWait(self.driver, timeout).until(EC.staleness_of(element))

    def attribute_value_is(self, locator, attribute, value):
        def _predicate(driver):
            element = driver.find_element(*locator)
            return value in element.get_attribute(attribute)
        return _predicate

    # def load_config(self, config_file):
    #     config = configparser.ConfigParser()
    #     config.read(config_file)
    #     return config

    def click_element(self, xpath):
        element = self.wait_for_clickable(By.XPATH, xpath)
        element.click()
        self.random_delay()

    def enter_text(self, xpath, text):
        field = self.wait_for_element(By.XPATH, xpath)
        field.send_keys(text)
        self.random_delay()

    def select_dropdown_by_text(self, xpath, text):
        dropdown = self.wait_for_element(By.XPATH, xpath)
        select = Select(dropdown)
        select.select_by_visible_text(text)
        self.random_delay()

    def upload_file(self, xpath, file_path):
        upload_field = self.wait_for_element(By.XPATH, xpath)
        upload_field.send_keys(file_path)
        self.random_delay()

    def switch_to_frame(self, xpath):
        frame = self.wait_for_element(By.XPATH, xpath)
        self.driver.switch_to.frame(frame)

    def select_dropdown_by_value(self, xpath, value):
        dropdown = self.wait_for_element(By.XPATH, xpath)
        select = Select(dropdown)
        select.select_by_value(value)
        self.random_delay()

class BangumiUploader(WebAutomationBase):
    LOGIN_BUTTON_XPATH = '//*[@id="main-menu-button"]'
    USERNAME_FIELD_XPATH = '/html/body/div[4]/md-dialog/md-content/form/md-input-group[1]/input'
    PASSWORD_FIELD_XPATH = '/html/body/div[4]/md-dialog/md-content/form/md-input-group[3]/input'
    SIGNIN_BUTTON_XPATH = '/html/body/div[4]/md-dialog/div/button[3]'
    PUBLISH_BUTTON_XPATH = '/html/body/div[1]/ul[2]/li[3]'
    OLD_VERSION_BUTTON_XPATH = '/html/body/div[4]/md-dialog/h2/span/button[2]/i'
    TITLE_FIELD_XPATH = '//*[@id="torrent-title"]'
    CAT_SELECT_DROPDOWN_XPATH = '/html/body/div[4]/md-dialog/md-content/div[1]/select'
    SOURCE_CODE_BUTTON_XPATH = '/html/body/div[4]/md-dialog/md-content/div[3]/ul/li[1]/a'
    SOURCE_CODE_FIELD_XPATH = '/html/body/div[4]/md-dialog/md-content/div[3]/textarea'
    VCB_IDENTITY_BUTTON_XPATH = '/html/body/div[4]/md-dialog/md-content/div[4]/md-list/md-item'
    TEAM_SYNC_CHECKBOX_XPATH = '/html/body/div[4]/md-dialog/md-content/md-checkbox'
    UPLOAD_BUTTON_XPATH = '/html/body/div[4]/md-dialog/md-content/input'
    FINAL_PUBLISH_BUTTON_XPATH = '/html/body/div[4]/md-dialog/div/button[2]'

    def run(self, username, password, title, cat, desc, torrent_path):
        self.get_driver()
        self.driver.get("https://bangumi.moe")

        # Close Tutorial
        self.wait_for_element(By.XPATH, "/html/body/div[6]/div/div[4]/a[1]")
        overlay = self.driver.find_element(By.XPATH, "/html/body/div[5]")
        self.click_element("/html/body/div[6]/div/div[4]/a[1]")
        self.wait_for_staleness(overlay)

        # Login
        self.click_element(self.LOGIN_BUTTON_XPATH)
        self.enter_text(self.USERNAME_FIELD_XPATH, username)
        self.enter_text(self.PASSWORD_FIELD_XPATH, password)
        self.click_element(self.SIGNIN_BUTTON_XPATH)

        # Form filling
        self.click_element(self.PUBLISH_BUTTON_XPATH)
        self.click_element(self.OLD_VERSION_BUTTON_XPATH)
        self.enter_text(self.TITLE_FIELD_XPATH, title)
        self.select_dropdown_by_value(self.CAT_SELECT_DROPDOWN_XPATH, cat)
        self.click_element(self.SOURCE_CODE_BUTTON_XPATH)
        self.enter_text(self.SOURCE_CODE_FIELD_XPATH, desc)
        self.upload_file(self.UPLOAD_BUTTON_XPATH, torrent_path)
        self.click_element(self.VCB_IDENTITY_BUTTON_XPATH)
        self.click_element(self.TEAM_SYNC_CHECKBOX_XPATH)
        print("Please click the Upload button manually.")
        input("Press Enter to quit...")
        self.close_driver()
        # self.click_element(self.FINAL_PUBLISH_BUTTON_XPATH)

class NyaaUploader(WebAutomationBase):
    CAPTCHA_IFRAME_XPATH = "/html/body/div[1]/form/div[3]/div/div/div/div/iframe"
    USERNAME_FIELD_XPATH = '//*[@id="username"]'
    PASSWORD_FIELD_XPATH = '//*[@id="password"]'
    CAPTCHA_ANCHOR_XPATH = '//*[@id="recaptcha-anchor"]'
    LOGIN_BUTTON_XPATH = '/html/body/div[1]/form/div[4]/div/input'
    UPLOAD_BUTTON_XPATH = '/html/body/nav/div/div[2]/ul[1]/li[1]/a'
    TORRENT_PATH_FIELD_XPATH = '//*[@id="torrent_file"]'
    TITLE_FIELD_XPATH = '//*[@id="display_name"]'
    CATEGORY_DROPDOWN_XPATH = '//*[@id="category"]'
    COMPLETE_BUTTON_XPATH = '/html/body/div/form/div[3]/div[2]/div[3]/label[2]'
    INFORMATION_FIELD_XPATH = '//*[@id="information"]'
    DESCRIPTION_FIELD_XPATH = '//*[@id="description"]'

    def run(self, username, password, title, cat, desc, torrent_path):
        self.get_driver()
        self.driver.get("https://nyaa.si/login")

        # Wait for page to load
        self.wait_for_element(By.XPATH, self.CAPTCHA_IFRAME_XPATH)

        # Login
        self.enter_text(self.USERNAME_FIELD_XPATH, username)
        self.enter_text(self.PASSWORD_FIELD_XPATH, password)
        print("Please finish captcha manually.")
        self.switch_to_frame(self.CAPTCHA_IFRAME_XPATH) # Switch to CAPTCHA iframe
        # Wait for CAPTCHA to be checked
        WebDriverWait(self.driver, 300).until(self.attribute_value_is(
            (By.XPATH, self.CAPTCHA_ANCHOR_XPATH), 'class', 'recaptcha-checkbox-checked')
        )
        self.driver.switch_to.default_content() # Switch back
        self.click_element(self.LOGIN_BUTTON_XPATH)

        # Form filling
        self.click_element(self.UPLOAD_BUTTON_XPATH)
        self.upload_file(self.TORRENT_PATH_FIELD_XPATH, torrent_path)
        self.enter_text(self.TITLE_FIELD_XPATH, title)
        self.select_dropdown_by_value(self.CATEGORY_DROPDOWN_XPATH, cat)
        self.click_element(self.COMPLETE_BUTTON_XPATH)
        self.enter_text(self.INFORMATION_FIELD_XPATH, "https://vcb-s.com/archives/138")
        self.enter_text(self.DESCRIPTION_FIELD_XPATH, desc)
        print("Please click the Upload button manually.")
        input("Press Enter to quit...")
        self.close_driver()
